/*
 * Copyright (c) 1994, 1997 Vectaport, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef dem_h
#define dem_h

/*
 * Generic Digital Elevation Model
 */

extern "C" {
#include "projects.h"
}

#include <stdio.h>

enum units { radians, feet, meters, arcseconds };

class DEM {
public:
    DEM();
    virtual ~DEM();

    int rows();
    int cols();
    void allocate(int rows, int cols);
    void value(int row, int col, unsigned short value);
    unsigned short value(int row, int col);

    void position (int row, int col, double& x, double& y);
    void projected_position (int row, int col, double& x, double& y);

    virtual int value_by_geopos(double x, double y, double& v);
    int value_by_utmpos(double ux, double uy, double& v);

    int tri3d_by_geopos(double x, double y,
			double& x1, double& y1, double& z1,
			double& x2, double& y2, double& z2,
			double& x3, double& y3, double& z3
			);
    int tri3d_by_utmpos(double x, double y,
			double& x1, double& y1, double& z1,
			double& x2, double& y2, double& z2,
			double& x3, double& y3, double& z3
			);

    int trimesh_by_geopos(double x0, double y0, double x1, double y1,
		       int& nt, double*& tx, double*& ty, double*& tz);
    int trimesh_by_utmpos(double x0, double y0, double x1, double y1,
		       int& nt, double*& tx, double*& ty, double*& tz);

    void projection(PJ*);
    PJ* projection() { return _proj; }
    void xyzdump(FILE*);

    virtual void read(char*) = 0;
    virtual void writerawpgm(char*) = 0;
    virtual int elev_max() = 0;

    double swe() { return _swe; }
    double swn() { return _swn; }
    double nwe() { return _nwe; }
    double nwn() { return _nwn; }
    double nee() { return _nee; }
    double nen() { return _nen; }
    double see() { return _see; }
    double sen() { return _sen; }
    int geowithin(double x, double y);
  units horizontal_units() { return grunits; }
  units vertical_units() { return elunits; }
  void horizontal_units(units gu) { grunits = gu; }
  void vertical_units(units eu) { elunits = eu; }
protected:
    double _swe, _swn, _nwe, _nwn, _nee, _nen, _see, _sen;
    double xres, yres, zres;
    units grunits;
    units elunits;
    int r, c;
    unsigned short* data;
    PJ* _proj;
};

#endif
