/*
 * Copyright (c) 1994 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef usgsdem_h
#define usgsdem_h

/*
 * USGS Digital Elevation Models - only handles 1-degree "DMA DEMs" for now
 */

#include <Dem/dem.h>

class UsgsDem : public DEM {
public:
    UsgsDem();
    ~UsgsDem();
    virtual void read(char* path);
    virtual void writerawpgm(char*);
    virtual int elev_max();
    virtual int elev_min();
    void print();

    enum QualityLevel { NULL_QLEVEL, DEM_1, DEM_2, DEM_3 };
    enum ReferenceSystem { GEOGRAPHIC, UTM, STATE_PLANE };
    enum UnitOfMeasure { RADIANS, FEET, METERS, ARC_SECONDS };
    enum VerticalDatum { NULL_VDATUM, LOCAL_MEAN_SEA_LEVEL, NGVD29, NAVD88 };
    enum HorizontalDatum { NULL_HDATUM, NAD27, WGS72, WGS84, NAD83,
			   OLD_HAWAII_DATUM, PUERTO_RICO_DATUM,
			   NAD83_PROVISIONAL };

    QualityLevel quality_level() { return level; }
    ReferenceSystem reference_system() { return refsys; }
    UnitOfMeasure ground_units() { return horz_units; }
    UnitOfMeasure elevation_units() { return vert_units; }
    VerticalDatum vertical_datum() { return vdatum; }
    HorizontalDatum horizontal_datum() { return hdatum; }

    const char* Name() { return name; }

    /* for 7.5 minute UTM DEMs */
    void utm_bbox(long& xmin, long& ymin, long& xmax, long& ymax) {
        xmin = xminutm; ymin = yminutm; xmax = xmaxutm; ymax = ymaxutm;
    }

    void reference_system(ReferenceSystem rs) { refsys = rs; }
    void ground_units(UnitOfMeasure um) { horz_units = um; }
    void elevation_units(UnitOfMeasure um) { vert_units = um; }
    void vertical_datum(VerticalDatum vd) { vdatum = vd; }
    void horizontal_datum(HorizontalDatum hd) { hdatum = hd; }

    void Name(char* n) { strcpy(name,n); }
  void res(double xr, double yr) {xres = xr; yres = yr;}
  double x_res() {return xres;}
  double y_res() {return yres;}

    /* for 7.5 minute UTM DEMs */
    void set_utm_bbox(long xmin, long ymin, long xmax, long ymax) {
        _swe = _nwe = xminutm = xmin;
	_swn = _sen = yminutm = ymin;
	_see = _nee = xmaxutm = xmax;
	_nwn = _nen = ymaxutm = ymax;
    }
    int value_by_utmpos(double ux, double uy, double& v);
  void value_at_utmpos(long ux, long uy, unsigned short v);

    int start_row(int col) { return startrow[col]; }
    int end_row(int col) { return endrow[col]; }
    virtual int value_by_geopos(double x, double y, double& v);
    void makexyz(int rsize, float sampdist, char* base = 0);
    int geowithin(double x, double y);
protected:
    QualityLevel level;
    ReferenceSystem refsys;
    UnitOfMeasure horz_units;
    UnitOfMeasure vert_units;
    VerticalDatum vdatum;
    HorizontalDatum hdatum;

    float emin, emax;
    long xminutm, xmaxutm, yminutm, ymaxutm;
    int* startrow;
    int* endrow;

    void fixd(char*);

    /* logical record A */
    char name[41];
    char comment[41];
    char processcode[2];
    char section[4];
    char mcorigin[5];
    char demlevel[7];
    char elevpattern[7];
    char referencesys[7];
    char refzone[7];
    char projparams[361];
    char coordunits[7];
    char elevunits[7];
    char nsides[7];
    char cornercoords[193];
    char fixedcornercoords[193];
    char elevminmax[49];
    char fixedelevminmax[49];
    char angle[25];
    char acccode[7];
    char resolutionunits[37];
    char numrowscolumns[13];
    /* old format stops here, they say */
    char largestcontour[6];
    char largestunits[2];
    char smallestcontour[6];
    char smallestunits[2];
    char sourcedate[5];
    char revdate[5];
    char insprevflag[2];
    char validationflag[2];
    char suspvoidflags[3];
    char verticaldatum[3];
    char horizontaldatum[3];
    char dataedition[5];
    char percentvoid[5];

    /* logical record C */
    char datumrmseavailable[7];
    char datumrmsexyz[19];
    char datumsamplesize[7];
    char demrmseavailable[7];
    char demrmsexyz[19];
    char demsamplesize[7];
};


#endif
