/************************************************************************
 *   MODULE  : LOSINIT.C
 *
 *   PURPOSE : main program initialization functions
 *
 *   AUTHOR  : Kevin Russo, SAIC (original, 1, 2)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   12 Aug 92     n/a
 *   1          24 OCT 92     overall program update
 *   2          01 Apr 93     update to program GUI
 *   3          18 Jun 93     update to XVT 3.0 and MUSE Beta Release 3
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

#if 0 /*XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN*/
#include <windows.h>
#endif

#if XVT_OS_ISUNIX == TRUE
#include <X11/Intrinsic.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
/* #include <Mrm/MrmAppl.h> */
#endif

/********************
 * global variables *
 ********************/

WIN_INFO wininfo;               /* non-muse window information structure */
MAP_TYPE mapinfo;               /* non-muse map information structure    */
SETTINGS_TYPE mapdefaults;      /* default settings                      */
SETTINGS_TYPE lastprod;         /* last product settings                 */
FONT_SIZE_TYPE fontsize;        /* holds info about system font          */
/* FONT XVT_FNTID xvt_R3FNT   small_font;                /* previously defined font in XVT 2.0    */
PREFS_TYPE sys_prefs;           /* System preferences                    */
PERCENT_TYPE percents;          /* perecnt products filled               */
TRIG_TYPE trig_table;           /* sine and cosine tables used for speed */
MUSEBYTE_ORDER byte_order;          /* used within the MUSE libraries        */
BOOLEAN no_process_vector;      /* used within the MUSE libraries        */
char dir_bin[100];              /* used within the MUSE libraries        */
short contrast = 0;             /* used to set map contrast              */
short old_contrast = 0;         /* used to set map contrast              */
BININFO bins[216];              /* used in contrast                      */

#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN
HPALETTE hPalette = NULL;       /* copy of the palette used by MUSE      */
BOOLEAN contrast_possible = TRUE; /* check if contrast control needed    */
#endif

#if XVT_OS_ISUNIX == TRUE
XColor    colrs[256];           /* colors palette definitions            */
Colormap  cmap;                 /* copy of the MUSE palette              */
#endif

/************************************************************************
 *   FUNCTION     : init_window_infomation
 *
 *   PURPOSE      : clears all windows to NULL and sets flags to FALSE
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_appl_init in module LOSAPPL.C
 *
 ************************************************************************/

#ifdef PROTO
void init_window_information(void)
#else
void init_window_information()
#endif
{
    wininfo.lookupTable501[0] = (WINDOW)NULL;
    wininfo.lookupTable502[0] = (WINDOW)NULL;
    wininfo.lookupTable503[0] = (WINDOW)NULL;
    wininfo.lookupTable504[0] = (WINDOW)NULL;

    contrast     = (short)ZERO;
    old_contrast = (short)ZERO;
}

/************************************************************************
 *   FUNCTION     : init_map_infomation
 *
 *   PURPOSE      : sets up (non-muse) map structure information 
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_file_menu in module LOSMENU1.C
 *
 ************************************************************************/           

#ifdef PROTO
void init_map_information(void)
#else
void init_map_information()
#endif
{
    extern SETTINGS_TYPE settings;    /* user entered settings           */
    
    mapinfo.cursor_pos.x = (GFLOAT)0.0;
    mapinfo.cursor_pos.y = (GFLOAT)0.0;    
    mapinfo.status[0]    = '\0';
    mapinfo.org.h        = 0;
    mapinfo.org.v        = 0;
    mapinfo.scale        = 1;
    mapinfo.t.x          = settings.target.loc.x;
    mapinfo.t.y          = settings.target.loc.y;
    mapinfo.o.x          = settings.observer.loc.x;
    mapinfo.o.y          = settings.observer.loc.y;
    mapinfo.boresight    = TRUE;
}

/************************************************************************
 *   FUNCTION     : init_mapdefault_settings
 *
 *   PURPOSE      : initializes default user settings 
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_file_menu in module LOSMENU1.C
 *
 ************************************************************************/           

#ifdef PROTO
void init_mapdefault_settings(void)
#else
void init_mapdefault_settings()
#endif
{
   mapdefaults.start           = (GFLOAT)MIN_ANGLE;
   mapdefaults.end             = (GFLOAT)MAX_ANGLE;
   mapdefaults.observer.loc.y  = (GFLOAT)ZERO;
   mapdefaults.observer.loc.x  = (GFLOAT)ZERO;
   mapdefaults.observer.height = (GFLOAT)ZERO;
   mapdefaults.target.loc.y    = (GFLOAT)ZERO;
   mapdefaults.target.loc.x    = (GFLOAT)ZERO;
   mapdefaults.target.height   = (GFLOAT)ZERO;
   mapdefaults.target.width    = (GFLOAT)ZERO;
   mapdefaults.range           = (GFLOAT)ZERO;

   mapdefaults.profile         = FALSE;
   mapdefaults.product         = FALSE;
   mapdefaults.prior_mask      = VISIBILITY;
   mapdefaults.prod_type       = VIS;


   mapinfo.o.x = (GFLOAT)ZERO;
   mapinfo.o.y = (GFLOAT)ZERO;
   mapinfo.t.x = (GFLOAT)ZERO;
   mapinfo.t.y = (GFLOAT)ZERO;
}

/************************************************************************
 *   FUNCTION     : init_handles
 *
 *   PURPOSE      : clears the product handles
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_appl_init in module LOSMENU1.C
 *
 ************************************************************************/           

#ifdef PROTO
void init_handles(void)
#else
void init_handles()
#endif
{
   extern HANDLES handle;           /* handles for all LOS calcs      */
   extern RASTER  elev_raster;      /* raster dted structure          */

   handle.dist = (GHANDLE)NULL;     /* distances handle               */
   handle.hgts = (GHANDLE)NULL;     /* heights handle                 */
   handle.xdim = (GHANDLE)NULL;     /* elevation x-dimension handle   */
   handle.ydim = (GHANDLE)NULL;     /* elevation y-dimension handle   */
   handle.prfl = (GHANDLE)NULL;     /* profile color handle           */

   elev_raster.data_matrix.ghandle = (GHANDLE)NULL;  /* dted handle   */
}

/************************************************************************
 *   FUNCTION     : init_percents
 *
 *   PURPOSE      : clears the product percents
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places
 *
 ************************************************************************/           

#ifdef PROTO
void init_percents(void)
#else
void init_percents()
#endif
{
   percents.visible      = (long)ZERO;
   percents.prob_visible = (long)ZERO;
   percents.hidden       = (long)ZERO;
   percents.prob_hidden  = (long)ZERO;
   percents.acuity       = (long)ZERO;

   percents.total        = (long)ZERO;
}

/************************************************************************
 *   FUNCTION     : init_trig_tables
 *
 *   PURPOSE      : initializes sine and cosine tables
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_appl_init in module PSCAPPL.C
 *
 ************************************************************************/

#ifdef PROTO
void init_trig_tables (void)
#else
void init_trig_tables ()
#endif
{
   GFLOAT value, *sine, *cosine;

   long loop;

   trig_table.shandle = (GHANDLE)xvt_gmem_alloc(3600 * sizeof(GFLOAT));
   if (trig_table.shandle == (GHANDLE)NULL)
      xvt_dm_post_fatal_exit("Unable to allocate memory for trig tables");

   trig_table.chandle = (GHANDLE)xvt_gmem_alloc(3600 * sizeof(GFLOAT));
   if (trig_table.chandle == (GHANDLE)NULL)
      xvt_dm_post_fatal_exit("Unable to allocate memory for trig tables");

   sine   = (GFLOAT *)xvt_gmem_lock(trig_table.shandle);
   cosine = (GFLOAT *)xvt_gmem_lock(trig_table.chandle);
   if ((sine == (GFLOAT *)NULL) || (cosine == (GFLOAT *)NULL))
      xvt_dm_post_fatal_exit("Unable to access trig table memory");

   for (loop = 0; loop < 3600; loop++)
   {
      value = (GFLOAT)loop * 0.10 / DEG_PER_RAD;

      sine[loop]   = sin(value);
      cosine[loop] = cos(value);
   }

   if (xvt_gmem_unlock(trig_table.shandle) == FALSE)
      xvt_dm_post_fatal_exit("Unable to unlock trig table memory");
   if (xvt_gmem_unlock(trig_table.chandle) == FALSE)
      xvt_dm_post_fatal_exit("Unable to unlock trig table memory");
}

/************************************************************************
 *   FUNCTION     : init_system_prefs
 *
 *   PURPOSE      : initializes system unit display prefs
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_appl_init in module LOSAPPL.C
 *
 ************************************************************************/

#ifdef PROTO
void init_system_prefs(void)
#else
void init_system_prefs()
#endif
{
   FILE *fileptr;

   short value;

   extern DIRECTORY home_dir;

   xvt_fsys_set_dir(home_dir.path);

   if (access ("los.ini", 0) != (int)ZERO)
   {
     /*      xvt_dm_post_note("Creating System Prefs File");*/

      sys_prefs.hgt_rel    = AGL;
      sys_prefs.loc_units  = DD_STYLE;
      sys_prefs.dist_units = METERS;
      sys_prefs.acuity     = TRUE;
      sys_prefs.obs_shape  = S_CIRCLE;
      sys_prefs.obs_col    = C_BLUE;
      sys_prefs.tgt_shape  = S_CIRCLE;
      sys_prefs.tgt_col    = C_RED;
      sys_prefs.bor_col    = C_YELLOW;

      fileptr = fopen ("los.ini", "wb");
      if (fileptr == (FILE *)NULL)
      {
	/*         xvt_dm_post_error("Unable to Create System Prefs File");*/
         return;
      }

      value = (short)sys_prefs.hgt_rel;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.loc_units;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.dist_units;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.acuity;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.obs_shape;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.obs_col;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.tgt_shape;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.tgt_col;
      fwrite (&value, sizeof(short), 1, fileptr);

      value = (short)sys_prefs.bor_col;
      fwrite (&value, sizeof(short), 1, fileptr);

      fclose (fileptr);
   }
   else
   {
      sys_prefs.hgt_rel    = AGL;
      sys_prefs.loc_units  = DD_STYLE;
      sys_prefs.dist_units = METERS;
      sys_prefs.acuity     = TRUE;
      sys_prefs.obs_shape  = S_CIRCLE;
      sys_prefs.obs_col    = C_BLUE;
      sys_prefs.tgt_shape  = S_CIRCLE;
      sys_prefs.tgt_col    = C_RED;
      sys_prefs.bor_col    = C_YELLOW;

      fileptr = fopen ("los.ini", "rb");
      if (fileptr == (FILE *)NULL)
      {
	/*         xvt_dm_post_note("Unable to Access System Prefs File - Using Defaults");*/
         return;
      }

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.hgt_rel = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.loc_units = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.dist_units = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.acuity = (BOOLEAN)value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.obs_shape = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.obs_col = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.tgt_shape = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.tgt_col = value;

      fread (&value, sizeof(short), 1, fileptr);
      sys_prefs.bor_col = value;

      fclose (fileptr);
   }
}
