/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcontour.h>
#include <Map/mapprim.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

ContourFeatureClass::ContourFeatureClass(const char* path)
: MapFeatureClass(nil, path)
{
}

ContourFeatureClass::~ContourFeatureClass() {
}

const char* ContourFeatureClass::description() {
    return nil;
}

MapFeature** ContourFeatureClass::make_features(int &nfeats) const {
    MapFeature** features = nil;
    FILE* f;
    DIR* dir;
    struct dirent* entry;
    struct stat status;
    int c = 0;

    dir = opendir(name());
    if (dir == NULL)
	return features;
    int nd = 0;
    while ((entry = readdir(dir)) != NULL)
	nd++;
    rewinddir(dir);
    features = (MapFeature**) new ContourFeature*[nd];
    while ((entry = readdir(dir)) != NULL) {
	if ((strcmp(entry->d_name,".") != 0) &&
	    (strcmp(entry->d_name,"..") != 0)) {
	    if (strstr(entry->d_name,".xyz")) {
		char path[256];
		strcpy(path, name());
		if (path[strlen(path)-1] != '/')
		    strcat(path, "/");
		strcat(path, entry->d_name);
		if ((f = fopen(path, "r")) == NULL)
		    continue;
		int i = 0;
		float x, y;
		int z;
		while (!feof(f)) {
		    if (fscanf(f, "%f %f %d", &x, &y, &z) != EOF)
			i++;
		}
		rewind(f);
		float* fx = new float[i];
		float* fy = new float[i];
		int j = 0;
		while (!feof(f)) {
		    fscanf(f, "%f %f %d", &fx[j], &fy[j], &z);
		    j++;
		}
		fclose(f);
		features[c] = new ContourFeature((ContourFeatureClass*)this, c, z);
		MapEdge* me = new MapEdge;
		me->insert_pointers(i,fx,fy,nil,true);
		features[c++]->edge_primitive(me);
	    }
	}
    }
    nfeats = c;
    closedir(dir);
    return features;
}

/*****************************************************************************/

ContourFeature::ContourFeature(ContourFeatureClass* fc, int id, int z)
: MapFeature(fc, id, MapFeature::LineType)
{
    _zval = z;
    _gr = nil;
}

ContourFeature::~ContourFeature() {
}

const char* ContourFeature::describe() const {
    static char buffer[BUFSIZ];
    sprintf(buffer, "Contour Line %d: Elevation %d", _id, _zval);
    return buffer;
}

void* ContourFeature::graphic() const {
    return _gr;
}

void ContourFeature::graphic(void* g) {
    _gr = g;
}
